#include "COMUtils.h"
#include <sapi.h>
#include <iostream>

const ULONGLONG grammarID1 = 1;
const WCHAR* ruleExitName = L"RegleQuitter";

const ULONGLONG grammarID2 = 2;
const WCHAR* ruleCountName = L"RegleCompter";



void GetText(SmartCOMPtr<ISpRecoContext> reco_context)
{
	const ULONG max_events = 10;
	ULONG nb_events;
	SPEVENT les_events[max_events]; // voir la dclaration de SPEVENT dans sapi.h est TRES instructif
	HRESULT hr;
	SmartCOMPtr<ISpRecoResult> reco_result;

	hr = reco_context->GetEvents(max_events, les_events, &nb_events);
	// Attention hr vaut S_FALSE si tout va bien mais
	// que le nombre d'vnements obtenus est infrieur
	// au nombre d'vnements demands
	if(!(hr == S_OK || hr == S_FALSE))
		COMHelper::TestOk(hr);
	std::cout << "Nb events: " << nb_events << std::endl;
	*(&reco_result) = reinterpret_cast<ISpRecoResult*>(les_events[0].lParam);

	WCHAR* le_texte;
	hr = reco_result->GetText(
		SP_GETWHOLEPHRASE, SP_GETWHOLEPHRASE, FALSE, &le_texte, NULL);
	COMHelper::TestOk(hr);
	std::wcout << "J'ai compris: " << le_texte << std::endl;
	::CoTaskMemFree(le_texte);
}

SmartCOMPtr<ISpRecoGrammar>
InstallGrammarCount(SmartCOMPtr<ISpRecoContext> reco_context)
{
}


SmartCOMPtr<ISpRecoGrammar>
InstallGrammarExit(SmartCOMPtr<ISpRecoContext> reco_context)
{
	HRESULT hr;
	SPSTATEHANDLE hStateQuitter;
	SmartCOMPtr<ISpRecoGrammar> reco_grammar;

	hr = reco_context->Pause(0);
	COMHelper::TestOk(hr);

	hr = reco_context->CreateGrammar(grammarID1, reinterpret_cast<ISpRecoGrammar **>(&reco_grammar));
	COMHelper::TestOk(hr);

	hr = reco_grammar->ResetGrammar(GetUserDefaultUILanguage());
	COMHelper::TestOk(hr);

	// Crer une rgle
	hr = reco_grammar->GetRule(ruleExitName, 0, SPRAF_TopLevel | SPRAF_Active, TRUE, &hStateQuitter);
	COMHelper::TestOk(hr);
	// Ajouter un mot
	hr = reco_grammar->AddWordTransition(
		hStateQuitter, NULL, L"Quitter", L" ", SPWT_LEXICAL, 1, NULL);
	COMHelper::TestOk(hr);

	// Trs important,
	// sinon rien est fait 
	hr = reco_grammar->Commit(0);
	COMHelper::TestOk(hr);
	hr = reco_grammar->SetRuleState(ruleExitName, 0, SPRS_ACTIVE),
	COMHelper::TestOk(hr);

	hr = reco_context->Resume(0);
	COMHelper::TestOk(hr);

	return reco_grammar;
}

void Run()
{
	HRESULT hr;
	SmartCOMPtr<ISpRecognizer> recognizer;
	SmartCOMPtr<ISpRecoContext> reco_context;
	SmartCOMPtr<ISpRecoGrammar> reco_grammar;
	HANDLE event;
	ULONGLONG interest;


	hr = ::CoCreateInstance(
		CLSID_SpSharedRecoContext, 
		NULL,
		CLSCTX_ALL,
		IID_ISpRecoContext,
		reinterpret_cast<void**>(&reco_context));
	COMHelper::TestOk(hr);
		
	reco_grammar = InstallGrammarExit(reco_context);	

	hr = reco_context->SetNotifyWin32Event();
	COMHelper::TestOk(hr);
	
	event = reco_context->GetNotifyEventHandle();
	if(event == INVALID_HANDLE_VALUE)
		COMHelper::TestOk(E_FAIL);
	
	std::cout << "Vous devez peut etre dire:"
		" commencer l'ecoute" << std::endl;
	// Si ci-dessous non invoque, le moteur de reconnaissance dmarre
	// normalement, mais nous ne sommes jamais notifis de rien
	interest = SPFEI(SPEI_RECOGNITION);
	hr = reco_context->SetInterest(interest, interest);
	COMHelper::TestOk(hr);

	// Attendre la reconnaissance d'un mot	
	HANDLE tab_handle[1];
	tab_handle[0] = event;
	::WaitForMultipleObjects(1, tab_handle, FALSE, INFINITE);
	std::cout << "J'ai entendu quelque chose" << std::endl;
	GetText(reco_context);
	// On dsactive la rgle
	hr = reco_grammar->SetRuleState(ruleExitName, 0, SPRS_INACTIVE),
	std::cout << "Maintenant je me bouche les oreilles" << std::endl;
	std::cout <<  "Bye, bye" << std::endl;
}

int main(int argc, char** argv)
{
	try
	{
		COMInitializer ci;
		Run();
	}
	catch(COMException ce)
	{
		std::cout << ce.raison() << std::endl;
		return EXIT_FAILURE;
	}
	return EXIT_SUCCESS;
}
